# File Maker for Photo Color Adjustment 16/8bit Version
# Ver. 3.00
# Silent & Verbose Version
# (c)2020-21 Ohnishi, Yasuo
from ij import IJ, ImagePlus 
from ij import plugin
from ij import WindowManager as WM
from ij.io import DirectoryChooser
from ij.io import OpenDialog
from ij.gui import GenericDialog
from ij.plugin import ChannelSplitter
from ij.plugin import Duplicator
from ij.process import ImageConverter
from loci.plugins import BF
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
    if Mode == "16-bit":
        Value = Value * 256
    elif Mode == "32-bit":
        Value = Value * 256 * 256
    else:
        Value = Value
    return Value
#
#####################################################
def RGBmanipulation(folder,filename,parm):
#  input Parameters
    if parm is not None:
 # if parm is not null object, then...
        Mode, parm_Binary, parm_Distance, parm_Dark, Show, parm_Prph, parm_FGTh = parm
        print(Mode, parm_Binary, parm_Distance, parm_Dark, Show, parm_Prph, parm_FGTh) 
#     Mode: Image bit depth 16-bit or 8-bit
#     Show: Show working windows or not
#     parm_Binary: Threshold of fore / background
#     parm_Distance: Lightness of Distance adjusting Layer
#     parm_Dark: Threshold of Dark part adjustment image
#     parm_Prph: Threshold of Periphery adjustment image
#     parm_FGTh: Threshold of Foreground mask

# Always Open file with Bio-format Plugin
    imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
    imp0 = imps[0] #imp0: Original Image 
    if Mode == "Auto":
        bitdepth = imp0.getBitDepth()
        if bitdepth == 8:
            Mode = "8-bit"
        elif bitdepth ==16:
            Mode = "16-bit"
        elif bitdepth ==32:
            Mode = "32-bit"
        else:
            Mode = "16-bit"
    if Show == "Yes":
 # Windows Showing Mode
        imp0.show()
#------
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
## R ch.
    impR =  impsCh[0]
    IJ.saveAsTiff(impR, folder + filename +"_R.tif")
## G ch.
    impG =  impsCh[1]
    IJ.saveAsTiff(impG, folder + filename +"_G.tif")
## B ch. & B x 0.3
    impB =  impsCh[2]
    IJ.saveAsTiff(impB, folder + filename +"_B.tif")
# Show Windows
    imp0.close()
    if Show == "Yes":
        impR.show()
        impG.show()
        impB.show()
##      Make impB03 as B x 0.3 Image
    impB03 = Duplicator().run(impB)
    IJ.run(impB03, "Multiply...", "value=0.3")
    if Show == "Yes":
        impB03.show()
# G ch. x 0.7
##        Make impG07 as G x 0.7 Image
    impG07 = Duplicator().run(impG)
    IJ.run(impG07, "Multiply...", "value=0.7")
    if Show == "Yes":
        impG07.show()
# B x 0.3 + G x 0.7
    ic = plugin.ImageCalculator()
    imp3 = ic.run("Add create", impB03, impG07) # No window made
    IJ.saveAsTiff(imp3, folder + filename +"_B_foreground.tif")
    imp3.close()
# (G ch.+30) x 0.7
##        Make impG30x07 as (G+30) x 0.7 Image
    impG30x07 = Duplicator().run(impG)
    if Show == "Yes":
        impG30x07.show()

    IJ.run(impG30x07, "Add...", "value=" + str(convValue(30, Mode))) 
    IJ.run(impG30x07, "Multiply...", "value=0.7")
## B x 0.3 + (G+30) x 0.7 + x(parm_Distance)
    ic = plugin.ImageCalculator()
    imp4 = ic.run("Add create", impB03, impG30x07) # No window made
#  add B x 0.3 + (G+30) x 0.7 + x
    IJ.run(imp4, "Add...", "value=" + str(convValue(parm_Distance, Mode))) # add Value parm_Distance
    IJ.saveAsTiff(imp4, folder + filename +"_B_Background.tif")
    imp4.close()
    if Show == "Yes":
  #Hide the image windows to close
        impG30x07.hide()
        impG07.hide()
        impB03.hide()
    impG30x07.close()
    impG07.close()
    impB03.close()
# G ch. x 0.5
#        Make impG05  as G x 0.5 Image
    impG05 = Duplicator().run(impG)
    if Show == "Yes":
        impG05.show()
    IJ.run(impG05, "Multiply...", "value=0.5")
# R ch. x 0.5
#        Make impR05  as R x 0.5 Image
    impR05 = Duplicator().run(impR)
    if Show == "Yes":
        impR05.show()
    IJ.run(impR05, "Multiply...", "value=0.5")
# B x 0.5 + R x 0.5
    ic = plugin.ImageCalculator()
    imp5 = ic.run("Add create", impR05, impG05) # No window made
    IJ.saveAsTiff(imp5, folder + filename +"_G+R.tif")
    if Show == "Yes": #Hide the image windows to close
        imp5.show()
        impR05.hide()
        impG05.hide()
    impR05.close()
    impG05.close()
# Periphery Adjustment Mask: B - ((R+G)/2-10)
    impGR = Duplicator().run(imp5) #imp5: (G+R)/2 image
    if Show == "Yes":
        impGR.show()
    IJ.run(impGR, "Subtract...", "value="+ str(convValue(parm_Prph, Mode))) # subtract Value parm_Prph /10
    ic = plugin.ImageCalculator()
    imp2GR = ic.run("Subtract create", impB, impGR) #impB - impGR:(G+R/2)
    if Show == "Yes":
        imp2GR.show()
    IJ.run(imp2GR, "Multiply...", "value=4") # Save the Subtracted image x 4
    IJ.saveAsTiff(imp2GR, folder + filename +"_Periphery_Adjst_Mask.tif") 
    if Show == "Yes":
 #Hide the image windows to close
        imp2GR.hide()
        impGR.hide()
    imp2GR.close()
 #Close the images
    impGR.close()
#
# Dark Parts 0 - parm_Dark
    imp6 = Duplicator().run(imp5) #imp5: (G+R)/2 image, imp6: dark part image
    if Show == "Yes":
        imp6.hide()
    IJ.run(imp6, "Add...", "value=" + str(convValue(255 - parm_Dark, Mode))) 
    IJ.run(imp6, "Subtract...", "value=" + str(convValue(255 - parm_Dark, Mode)))
    IJ.saveAsTiff(imp6, folder + filename +"_Dark.tif")
    if Show == "Yes":
 #Hide the image windows to close
        imp5.hide()
        imp6.hide()
    imp5.close()
 #Close the images
    imp6.close()
# R + parm_FGTh(128)
    imp8 = Duplicator().run(impR)
    if Show == "Yes":
        imp8.show()
    IJ.run(imp8, "Add...", "value=" + str(convValue(parm_FGTh, Mode)))
    IJ.saveAsTiff(imp8, folder + filename +"_R+128.tif")
    if Show == "Yes":
        imp8.hide()
    imp8.close()
# Make Binary Mask: Threshold R parm_Binary
    imp9 = Duplicator().run(impR)
    if Show == "Yes":
        imp9.show()
    imp9.getProcessor().threshold(int(convValue(parm_Binary, Mode))) # Make binary Mask
    if Mode <> "8-bit":
        IJ.run(imp9, "Multiply...", "value=" + str(convValue(1, Mode))) # 16, 32bit image > Multiply 256 ^ 1 or 2
        IJ.run(imp9, "Add...", "value=" + str(convValue(1, Mode) - 1)) # if 16-bit > make Max Value 65535
    imp9.updateAndDraw()
    IJ.saveAsTiff(imp9, folder + filename +"_Binary_Mask.tif")
    if Show == "Yes":
 #Hide the imp9 window to close
        imp9.hide()
    imp9.close()
    impR.close()
    impG.close()
    impB.close()
#####################################################

# Get parameters from the input DialogBox 
def getParameters():
	gd = GenericDialog("Enter Parameters")
	mode = ["Auto","16-bit","8-bit"]
	gd.addChoice("Image bit depth", mode, mode[0])
	FGTh = ["128", "48"]
	gd.addChoice("Foreground Mask Threshold (128 or 48)", FGTh, FGTh[0])
	gd.addNumericField("Background Mask Threshold (0 to 255)", 128, 0)
	gd.addNumericField("Background Lightness (Max to 100)", 0, 0)
	gd.addNumericField("Dark part Threshold (0 to 255)", 50, 0)
	gd.addNumericField("Periphery Adjst Threshold (-10 to 10)", 10, 0)
	showWin = ["Yes","No"]
	gd.addChoice("Show working windows?", showWin, showWin[0])
	gd.showDialog()
	parm_Mode   = gd.getNextChoice()
	parm_FGTh   = int(gd.getNextChoice())
	parm_Binary   = gd.getNextNumber()
	parm_Distance = gd.getNextNumber()
	parm_Dark = gd.getNextNumber()
	parm_Prph = gd.getNextNumber()
	parm_Show   = gd.getNextChoice()
	if parm_Binary > 255: #Set Max Value
	    parm_Binary = 255
	if parm_Distance > 100:
	    parm_Binary = 100
	if parm_Dark > 255: #Set Max Value
	    parm_Dark = 255

	if gd.wasCanceled():  
		parm_Mode   = "Stop"
		IJ.log("Canceled")
		return str(parm_Mode), parm_Binary, parm_Distance, parm_Dark,str(parm_Show), parm_Prph, parm_FGTh
		
	return str(parm_Mode), parm_Binary, parm_Distance, parm_Dark, str(parm_Show), parm_Prph, parm_FGTh

# Main
od = OpenDialog("Choose a file")
folder = od.getDirectory()
file = od.getFileName()
path = folder + file
IJ.log(path)
IJ.log(folder)
IJ.log(file)
parm = getParameters()
print ("Parm[0]=", parm[0])
if parm[0] <> "Stop":
	RGBmanipulation(folder,file,parm)
	IJ.log("Finish")
	IJ.run("Quit")
